use super::{
    token::Token,
    utils::{describe_line, parse_line},
    Pair,
};

#[derive(Debug, Clone)]
#[allow(dead_code)]
pub enum Command {
    Say { name: Option<String>, text: String },
    Eat { food: String, politely: bool },
    Define { variable: String, value: Token },
    Assign { variable: String, value: Token },
}

pub fn parse_command(pair: Pair) -> Command {
    use Command::*;
    use Token::*;
    let line = parse_line(pair);
    macro_rules! unknown {
        () => {
            panic!("Unknown command {}", describe_line(&line))
        };
    }
    match line.as_slice() {
        [Str(text)] => Say {
            name: None,
            text: text.clone(),
        },
        [Str(name), Str(text)] => Say {
            name: Some(name.clone()),
            text: text.clone(),
        },
        [Keyword(keyword), Keyword(variable), Keyword(equals), value]
            if keyword.eq("define") && equals.eq("=") =>
        {
            Define {
                variable: variable.clone(),
                value: value.clone(),
            }
        },
        [Keyword(keyword), Keyword(variable), Keyword(equals), value]
            if keyword.eq("$") && equals.eq("=") =>
        {
            Assign {
                variable: variable.clone(),
                value: value.clone(),
            }
        },
        [Keyword(keyword), Str(food), tail @ ..] if keyword.eq("eat") => Eat {
            food: food.to_owned(),
            politely: match tail {
                [Boolean(politely)] => *politely,
                _ => unknown!(),
            },
        },
        _ => unknown!(),
    }
}
